// --- [ atdb081.c ] ----------------------------------------------------------
//
// tab = 3
// ----------------------------------------------------------------------------

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/eeprom.h>
#include <inttypes.h>
#include "at45db081.h"
#include "spi_hard.h"
#include "global.h"

uint8_t df_adr[3];								// die Adresse in Bytes aufbereitet

// --- lokale Funktionen ------------------------------------------------------
void DF_make_adr(uint16_t page, uint16_t byte_adr);

// ----------------------------------------------------------------------------
// Schreibt eine komplette Page aus einem der beiden internen Puffer
// (in zwei Schritten) an DF.
// Zuerst werden die Daten in einen der beide DF-Buffer geschickt,
// dann wird das Kommando gegeben, den DF-Buffer in den Speicher des DF
// zu transferieren.
// Mit dem Parameter idx wird mitgeteilt, in welchen der beiden internen Puffer die zu
// schreibenden Daten sich befinden
// Da idx 0oder 1 betrgt, wird er auch gleich zur Auswahl der beiden Puffer auf dem DF
// herangezogen: die Daten werden abwechselnd in den Zwischenpuffer 1 bzw. 2 geschrieben
void DF_write_page(uint8_t idx)
{
uint8_t i = 0;

DF_make_adr(df_next_page, 0);					// fr beide Aktionen die Speicheradresse ermitteln
														// den Header vorbereiten
df_header[P_FLAG]      = 0;					// ein gelschtes Byte hat den Wert 255 !
df_header[P_SIZE]      = bytes_received;	// Pagesize; 0	bedeutet, dass 1 Byte eingegangen ist !
														// der Index zeigt bereits auf das nchste Elemente !
df_header[P_ID_HIGH]   = (uint8_t) (file_id >> 8);	// File-ID
df_header[P_ID_LOW]    = (uint8_t)  file_id;
df_header[P_PAGE_HIGH] = (uint8_t) (file_page_cnt >> 8);// Pagecounter fr Datei
df_header[P_PAGE_LOW]  = (uint8_t)  file_page_cnt;
df_header[P_TIME_HIGH] = (uint8_t) (time >> 8 );	// die interne Zeit
df_header[P_TIME_LOW]  = (uint8_t)  time;
/* ------------------------------------*/ // Daten in den Buffer

// Im letzten Datensatz unbenutzte Bytes mit dem Wert 255 auffllen
// lt. Dokumentation sollten unbenutzte Bytes mit 255 beschrieben werden
if (bytes_received < 255)						// wenn weniger als 256 Byte in der Page vorhanden sind
	{													// die freien Bytes auf 255 setzen
	for (i = (bytes_received + 1); i; i++) rx_buffer[idx][i] = 255;
	}

SET_PIN_CS;											// cs setzen
if (idx)
	{
	SPI_write(BUFFER_1_WRITE);					// Kommando via Buffer 1 schreiben
	}

else
	{
	SPI_write(BUFFER_2_WRITE);					// Kommando via Buffer 2 schreiben
	}
SPI_write(0);										// Adr[] ist hier ohne Bedeutung
SPI_write(df_adr[1]);
SPI_write(df_adr[2]);

i = 0;												// die Daten schreiben
do	{ SPI_write(rx_buffer[idx][i++]);		// das Array mit 256 Byte
	} while (i);
i = 0;												// dann den Header mit 8 Byte
do	{ SPI_write(df_header[i++]);
	} while (i < 8);

CLEAR_PIN_CS;


/* ------------------------------------*/ // Daten vom Buffer in den Speicher

while (!(DF_device_ready()));					// warten, bis DF bereit

SET_PIN_CS;											// cs setzen
if (idx)
	{													// Kommando Buffer 1 to Memory
	SPI_write(B1_TO_MM_PAGE_PROG_WITHOUT_ERASE);
	}

else
	{													// Kommando Buffer 2 to Memory
	SPI_write(B2_TO_MM_PAGE_PROG_WITHOUT_ERASE);
	}
SPI_write(df_adr[0]);
SPI_write(df_adr[1]);
SPI_write(0);										// Adr[2] ist hier ohne Bedeutung

CLEAR_PIN_CS;

/* ------------------------------------*/ // die nchste Page lschen

#ifdef NO_OVERWRITE								// siehe global.h
session_pages++;									// Zhler fr die geschriebenen Pages 
														// innerhalb einer Sitzung, wenn
if (session_pages & (!(PAGE_MASK)))			// mehr als 4096 Pages geschrieben
	{													// dann keine Daten berschreiben
	LED1_ON;
	LED2_ON;											// alle LEDs aus
	LED3_ON;
	while(1)											// alle drei LEDs im Gleichtakt toggeln
		{
		LED_timeout = 7;
		while (LED_timeout);
		LED1_TOG;
		LED2_TOG;
		LED3_TOG;
		}
	}
#endif

df_next_page++;									// Nchste Page
df_next_page &= PAGE_MASK;
#ifdef NO_OVERRUN									// kein Indexberlauf am Ende des Speichers
if (!(df_next_page))								// wenn der Index auf 0 bergelauften ist
	LED1_ON;
	LED2_OFF;										// LED1/3 an, LED2 aus
	LED3_ON;
	while(1)											// alle drei LEDs im Gleichtakt toggeln
		{
		LED_timeout = 7;
		while (LED_timeout);
		LED1_TOG;
		LED2_TOG;
		LED3_TOG;
		}
	}
#endif

/* ------------------------------------*/ // die nchste Page lschen
file_page_cnt++;									// Zhler fr die Pages einer Datei
DF_erase_page(df_next_page);					// die nchste Page vorbereiten/lschen
														// df_next_page zeigt immer auf die
}														// folgende (noch leere) Page

// ----------------------------------------------------------------------------
// Lscht eine Page als Vorbereitung vor dem Schreiben von Daten
// hier muss die Page-nr per Value bergeben werden
// dauert 30ms !!
void DF_erase_page(uint16_t page)
{
while (!(DF_device_ready()));					// warten, bis DF bereit
DF_make_adr(page, 0);							// die Adresse berechnen
									
SET_PIN_CS;											// cs setzen

SPI_write(PAGE_ERASE);							// Kommando schreiben
SPI_write(df_adr[0]);
SPI_write(df_adr[1]);
SPI_write(0);										// hier ist kein Wert erforderlich

CLEAR_PIN_CS;										// cs lschen
}

// ----------------------------------------------------------------------------
// gibt TRUE zurck, wenn das Gert bereit ist

uint8_t DF_device_ready(void)
{
uint8_t status;

SET_PIN_CS;											// cs setzen

SPI_write(STATUS_REGISTER);					// Kommando schreiben
status = SPI_read();								// Antwort abholen

CLEAR_PIN_CS;										// cs lschen

// damit ein 0xFF (eine ausbleibende Antwort) nicht zu einem TRUE fhrt, weitere Bits auswerten
//return(status & 0x80);						// Bit.7 maskieren
status = status & 0b10111100;					// Achtung - Gerteabhngig/Speicherabhngig !!!
if (status == 164) return(TRUE);				// Achtung - Gerteabhngig/Speicherabhngig !!!
														// ergibt sich aus 128 + chipspezifischen Bits
return(FALSE );
}

// ----------------------------------------------------------------------------
// schreibt die Device_ID nach df_buffer[]
void DF_read_device_id(void)
{
uint8_t i;

SET_PIN_CS;											// cs setzen

SPI_write(DEVICE_ID);							// Kommando schreiben

for (i = 0; i < 6; i++)
	{
	df_buffer[i] = SPI_read();					// ID abholen
	}

CLEAR_PIN_CS;										// cs lschen
}

// ----------------------------------------------------------------------------
// gibt TRUE zurck, wenn beim letzten Vergleich Buffer-Memory einen Fehler auftrat
uint8_t DF_compare_failed(void)
{
uint8_t status;

SET_PIN_CS;											// cs setzen

SPI_write(STATUS_REGISTER);					// Kommando schreiben
status = SPI_read();								// Antwort abholen

CLEAR_PIN_CS;										// cs lschen

return(status & 0x40);							// Bit.6 maskieren
}

// ----------------------------------------------------------------------------
// Durchsucht den gesamten DF, ermittelt, wieviele Blcke frei / belegt sind,
// die Anzahl der Dateien auf dem DF,  die hchste Datei-ID,
// die Pagenummer fr den nchsten zu speichernde Page.
// Dazu werden aus allen Pages die Bytes 256 bis 264 eingelesen
// ein belegter Block
// Byte.0 = P_FLAG      = 0, wenn die page belegt ist, sonst 255
// Byte.1 = P_SIZE      = Anzahl der Bytes in der Page
// Byte.2 = P_ID_HIGH   = Highbyte file_id
// Byte.3 = P_ID_LOW    = lowbyte file_id
// Byte.4 = P_PAGE_HIGH = Highbyte page_counter
// Byte.5 = P_PAGE_LOW  = Lowbyte page_counter
// Byte.6 = P_TIME_HIGH = Highbyte Zeit
// Byte.7 = P_TIME_LOW  = Lowbyte Zeit

void DF_check(void)
{
uint8_t start_found = FALSE;
uint16_t ii, cc;

df_next_page = 0;									// nchste frei Page
file_id = 0;										// nchste File-Id ("Dateiname")

LED1_ON;												// LED1/2 unterschiedlich Schalten
LED2_OFF;

for (ii = 0; ii < (DF_PAGES_ON_CHIP - 1); ii++)
	{
	DF_read_byte(ii, 256, 4);					// 4 Byte nach df_buffer einlesen

	if (df_buffer[P_FLAG] == 0)				// wenn die Page benutzt ist
		{
		cc = ((df_buffer[P_ID_HIGH] << 8) + df_buffer[P_ID_LOW]);
		if (cc > file_id) file_id = cc;		// die hchste File_id merken
		}

	else 												// wenn der Block nicht benutzt ist
		{
		if (!(start_found))						// wenn noch keine leere Page gefunden
			{
			start_found = TRUE;					// damit nachfolgende Seiten ignoriert werden
			df_next_page = ii;					// das ist diese die nchste freie Page
			}
		}

	if (LED_timeout == 0)
		{
		LED_timeout = 3;							// schnelles Blinken
		LED1_TOG;									// beide LEDs togglen
		LED2_TOG;
		}
	}
}

// ----------------------------------------------------------------------------
// liest 'cnt'-Bytes ab der angegebenen Adresse aus dem DF
// und schreibt die Daten nach df_buffer[]

void DF_read_byte(uint16_t page, uint16_t byte_adr, uint8_t cnt)
{
uint8_t i = 0;

DF_make_adr(page, byte_adr);
SET_PIN_CS;											// cs setzen

SPI_write(CONT_READ_LOW);						// Kommando schreiben
SPI_write(df_adr[0]);
SPI_write(df_adr[1]);
SPI_write(df_adr[2]);

if (cnt == 255)
	{
	do	{ df_buffer[i++] = SPI_read(); } while (i);
	}
else
	{
	do	{ df_buffer[i++] = SPI_read(); } while (i <= cnt);
	}

CLEAR_PIN_CS;
}

// ----------------------------------------------------------------------------
// liest 256 Byte einer Page beginnend mit der angegebenen Byte-Adresse aus dem DF
// und schreibt die Daten nach df_buffer[]

void DF_read_page(uint16_t page, uint16_t byte_adr)
{
uint8_t i = 0;

DF_make_adr(page, byte_adr);					// die Adresse umrechnen

SET_PIN_CS;											// cs setzen

SPI_write(MAIN_MEMORY_PAGE_READ);			// Kommando schreiben
SPI_write(df_adr[0]);
SPI_write(df_adr[1]);
SPI_write(df_adr[2]);

SPI_write(0);										// 4 dont care Bytes
SPI_write(0);
SPI_write(0);
SPI_write(0);

do	{ df_buffer[i++] = SPI_read();			// Daten nach sf_buffer[] schreiben
	} while (i);

CLEAR_PIN_CS;
}

// ----------------------------------------------------------------------------
// Bildet aus den Variablen 'page' und 'byte_adr'
// die zur Adressierung im 264-Byte Modus notwendigen Adressbytes
void DF_make_adr(uint16_t page, uint16_t byte_adr)
{
page = page << 1;									// Page um 1 Bit nach links schieben

df_adr[0]  = (uint8_t) (page >> 8);
df_adr[1]  = (uint8_t)  page;
df_adr[1] += (uint8_t) (byte_adr >> 8);	// das obere Bit der Adresse addieren
df_adr[2]  = (uint8_t)  byte_adr;
}

// ----------------------------------------------------------------------------
// lscht den gesamten Speicher,
// wartet bis zum Abschluss der Aktion, dabei blinken die beiden
// LEDs schnell im Gegentakt.
void DF_erase_chip(void)
{
SET_PIN_CS;											// cs setzen

SPI_write(0xC7);									// feste Sequenz schreiben
SPI_write(0x94);
SPI_write(0x80);
SPI_write(0x9A);

CLEAR_PIN_CS;

LED1_ON;												// LEDs fr Gegentakt vorbereiten
LED2_OFF;

LED_timeout = 7;									// warten bis zum Abschluss des Lschens
while (!(DF_device_ready()))
	{
	if (LED_timeout == 0)						// dabei die LED schnell im Gegentakt blinken lassen
		{
		LED_timeout = 7;
		LED1_TOG;
		LED2_TOG;
		}
	}
}

// --- [ eof ] ----------------------------------------------------------------