// --- [ rs232.c ] -----------------------------------------------------------
// fr Senden und Empfangen via RS232

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <stdlib.h>  
#include "rs232.h"
#include "global.h"

void print_temperatur(int8_t temphi, uint8_t templo);

// ----------------------------------------------------------------------------
// 
void RS232_init(void)
{
/* ATMEGA8
UBRRL = TEILER;									// siehe rs232.h
UBRRH = 0;
UCSRB = (0<<RXCIE | 1<<TXEN | 0<<RXEN);
//UCSRC = (1<<URSEL | 0<<UMSEL) | (1<<UCSZ1) | (1<<UCSZ0);
// URSEL - Achtung bei ATmega 8
UCSRC = (0<<UMSEL) | (1<<UCSZ1) | (1<<UCSZ0);
*/


// ATMEGA88
UBRR0L = TEILER;									// siehe rs232.h
UBRR0H = 0;
UCSR0C = (0<<UMSEL01) | (1<<UCSZ01) | (1<<UCSZ00);
// RX/Tx und Rx_Interrupt enablen
// UCSR0B =	(1<<RXCIE0 | 0<<TXCIE0 | 0<<UDRIE0 | 1<<RXEN0 | 1<<TXEN0);
// hier nur zum Senden vorbereiten
UCSR0B =	(0<<RXCIE0 | 0<<TXCIE0 | 0<<UDRIE0 | 0<<RXEN0 | 1<<TXEN0);
}


/*
// ----------------------------------------------------------------------------
// legt ein via RS232 eingegangenes Byte im public Ringpuffer rx_buffer ab.
// prft nicht, ob der Puffer von hinten her berschrieben wird !
ISR (USART_RX_vect)
{
rx_buffer[rx_buffer_ptr][rx_idx++] = UDR0;

// das Array ist 256 Byte lang, eine berlaufkontrolle ist also nicht erforderlich
// dennoch muss signalisiert werden, dass die Daten nun gesendet werden sollen

if (rx_idx == 0)
	{
	rx_flag = TRUE;							// Flag fr vollstndigen Block setzen
	rx_buffer_ptr ^= 1;						// den Pointer auf den anderen EmpfangsBuffer setzen
//	bytes_received = 255;					// ist hier berflssig, kann am Flag erkannt werden	
	}
}   
*/

// ----------------------------------------------------------------------------
// gibt den Header einer Page aus
// die Pagenummer ist lokal und wird bergeben 
void RS232_print_header(uint16_t page)
{
put_c(13);
put_s("page#: ");
put_s(utoa(page, msg,10));
put_s(", bytes: ");
put_s(utoa(df_buffer[1], msg, 10));
put_s(", file#: ");
put_s(utoa(((df_buffer[2]<< 8) + df_buffer[3]), msg, 10));
put_s(", #: ");
put_s(utoa(((df_buffer[4]<< 8) + df_buffer[5]), msg, 10));
put_s(", time: ");
put_s(utoa(((df_buffer[6]<< 8) + df_buffer[7]), msg, 10));
put_c(13);
put_c(13);
}

// ----------------------------------------------------------------------------
// gibt den gesamten Puffer von bis zu 256 Byte aus
void RS232_print_page(uint8_t cnt)
{
uint8_t i, j;

/*
i = 0;
if (cnt == 255)								// hier wird nur ein Speicherauszug gedruckt
	{												// fr ASCII-Datei
	do	{ put_c(buffer[i++]); } while (i);
	}
else
	{
	do { put_c(buffer[i++]); } while (i <= cnt);
	}
} */

/*
i = 0;
if (cnt == 255)								// hier wird nur ein Speicherauszug gedruckt
	{												// fr Binaer-Datei
	j = 0;
	do	{ 
		if (df_buffer[i] < 16) put_c('0');
		put_s(utoa((df_buffer[i++]), msg, 16)); 
		j++;
		if (j % 16 == 0) put_c(13);
		else 	put_c(' ');
		} while (i);
	}
else
	{
	j = 0;
	do { 
		if (df_buffer[i] < 16) put_c('0');
		put_s(utoa((df_buffer[i++]), msg, 16));
		j++;
		if (j % 16 == 0) put_c(13);
		else 	put_c(' ');
		} while (i <= cnt);
	}
*/ 

// Hier wird der Inhalt einer Page formatiert ausgegeben.
// Aufgezeichnet sind die Daten von 4 Temperatursensoren,
// je Sensor werden pro Messung 4 Byte gespeichert.

// Die Daten sind jeweils in vier Byte-Gruppen aufgezeichnet:
// Byte[0] = Sensornummer
// Byte[1]   ohne Bedeutung
// Byte[2] = Ganzahl der Temperatur als INT8
// Byte[3] = Nachkommastelle als INT8 

j = 0;
for (i = 0; i < cnt; i += 4)
	{
		
	put_c(df_buffer[i] + 'A');				// Sensor_Id mit Grobuchstaben
	put_c(':');
	put_c(' ');
	if (df_buffer[i+3] == 255)				// wenn kein Sensor vorhanden war, dann
		{
		put_c('-');								// "---.-" ausgeben
		put_c('-');
		put_c('-');
		put_c('.');
		put_c('-');
		}
		
	else
		{
		print_temperatur(df_buffer[i+2], df_buffer[i+3]);
		
//		put_s(" -> ");							// hier werden die Rohdaten der Temperatur gedruckt
//		put_s(utoa(df_buffer[i+2], msg,2));
//		put_s(" _ ");
//		put_s(utoa((df_buffer[i+3] >> 4), msg,2));

		}
	j++;
	if (j % 4)									// Nach jedem Messwert ein Komma ausgeben
		{
		put_c(',');
		put_c(' ');
		}
	else put_c(13);							// Nach dem 4. Messwert ein RETURN
	
	if (i > 251) return;						// damit keine Endlosschleife auftritt
	}												// wegen berlauf ber 255 

} 

// ---------------------------------------------------------------------------
// baut eine formatierte Temperaturanzeige auf.
// Der Text hat ist immer 5 Zeichen lang, ggf. wird ein fhrendes Leerzeichen vorangestellt
void print_temperatur(int8_t temphi, uint8_t templo)
{
uint16_t temp16;

if (temphi >= 10)									// ein "+" voranschreiben, wenn >= 10
	{
	put_c('+');
	}

else if (temphi >= 0)							// ein " +" voranschreiben, wenn >= 0
	{
	put_c(' ');
	put_c('+');
	}

else if ((temphi > -2) && templo)			// fr -0.x Grad, hier negatives Vorzeichen setzen
	{													// aber nicht bei genau -1
	put_c(' ');
	put_c('-');
	}

else if (temphi > -10) 							// fr 0 bis -9 ein Leerzeichen ausgeben
	{
	put_c(' ');										// das Vorzeichen kommt von itoa()
	}

templo &= 0xF0;									// die Werte stehen im upper nibble
if (temphi < 0) 									// bei negativen Werten (Vorkomma) wirds kompliziert ...
	{
	templo = 0 - templo;							//	bei negativen Temperaturen Zweierkomplement bilden
	if (templo & 0xF0)							// wenn Nachkommastellen vorhanden
		{												// dann eine 1 zur Vorkommastelle addieren 
		temphi++;									// z.B. -3 + 1 = -2 (und daran kommen die Nachkommastellen)
		}
	}

temp16 = (uint16_t) templo * (640 / 16);
temp16 /= 1024;									// insgesamt mit 0.625 multiplizieren
	
put_s(itoa(temphi, msg, 10));					// das negative Vorzeichen kommt von hier
put_c('.');
put_c((int8_t) temp16 + 0x30);				// die Nachkommastelle besteht nur aus 1 Ziffer
}

// ----------------------------------------------------------------------------
// ein einzelnes Zeichen auf COM ausgeben
void put_c(uint8_t Zeichen)
{
while(!(UCSR0A & (1<<UDRE0)));
UDR0 = Zeichen;
}

// ----------------------------------------------------------------------------
// einen String auf COM ausgeben
void put_s(char *s)
{
while(*s) put_c(*s++);
}

/*
// ----------------------------------------------------------------------------
// gibt einen String aus dem Flash aus
void puts_p(const char *progmem_s)
{
register char c;
while ( (c = pgm_read_byte(progmem_s++)) )
	{
   put_c(c);
	}
}
*/
// --- [ eof ] ----------------------------------------------------------------